#
#  APRS4R - a ruby based aprs gateway/digipeater
#  Copyright (C) 2006 by Michael Conrad <do5mc@friggleware.de>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
#

require 'aprs4r/SocketConfiguration'
require 'aprs4r/ConfigurationAttribute'

require 'aprs4r/KISSDeviceConfiguration'
require 'aprs4r/UDPDeviceConfiguration'


module APRS4R

  class AX25SocketConfiguration < SocketConfiguration

    attr_accessor :call, :device, :port, :baudrate, :mode, :speed, :duplicatePeriod, :timeout, :parameters


    def to_yaml_properties
      super + %w{ @call @device @port @baudrate @mode @speed @duplicatePeriod @timeout @parameters}
    end


    def attributes
      attributes = [
                    ConfigurationAttribute.new( "call",            "text",    true, 10, "MYCALL",              "Rufzeichen"),
                    ConfigurationAttribute.new( "mode",            "text",    true, 15, "kiss/tnc2/northlink", "Modus"),
                    ConfigurationAttribute.new( "duplicatePeriod", "number",  true, 10, 15,                    "Duplikatsinterval"),
                    ConfigurationAttribute.new( "parameters",      "complex", true, 20, KISSDeviceConfiguration.new, "Parameter"),

                    ConfigurationAttribute.new( "device",          "text",    true, 10, "/dev/usb/tts/0",      "Gert", nil, true),
                    ConfigurationAttribute.new( "port",            "number",  false,  6, 0,                     "Port (Multi-TNC)", nil, true),
                    ConfigurationAttribute.new( "baudrate",        "number",  false,  6, 9600,                  "Baudrate", nil, true),
                    ConfigurationAttribute.new( "speed",           "number",  false,  6, 1200,                  "Geschwindigkeit", nil, true),
                    ConfigurationAttribute.new( "timeout",         "number",  false,  6, 300,                   "Timeout", nil, true)
                   ]

      return super + attributes
    end

    
    def update_configuration( old_version, new_version) 

      if old_version == "undefined" 

        if @parameters.nil?

          printf( "AX25Socket: converting KISS settings...")
          
          kiss = KISSDeviceConfiguration.new

          kiss.name = "kiss port" 
          kiss.type = "KISSDevice"
          kiss.enable = @enable

          # use existing settings
          kiss.device = @device
          kiss.baudrate = @baudrate
          kiss.port = @port
          kiss.speed = @speed
          kiss.timeout = @timeout

          # use default settings
          kiss.txdelay = nil
          kiss.persistence = nil
          kiss.slottime = nil
          kiss.duplex = nil

          @parameters = kiss

          @device = nil
          @port = nil
          @baudrate = nil
          @speed = nil
          @timeout = nil

          printf( " successful\n")

          return true
        end
        
      end

      return false
    end

  end

end
