#
#  APRS4R - a ruby based aprs gateway/digipeater
#  Copyright (C) 2008 by Michael Conrad <do5mc@friggleware.de>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
#

require 'socket'
require 'thread'
require 'timeout'
require 'io/nonblock'

require 'aprs4r/APRS4RLogger'

require 'aprs4r/APRSCall'
require 'aprs4r/Socket'


module APRS4R

  class FileSocket < Socket

    @logger = APRS4RLogger.get_logger( "FileSocket")
    
    
    def initialize( configuration)
      logger.info( "initialize( configuration)")
      
      super( configuration)

      @file = configuration.file
      @rate = configuration.rate.to_i
      @offset = configuration.offset.to_i
      
      @delay = 0 
      @delay = 1.0 / @rate if @rate > 0
      @first_read = true 
      
      @counter = 0 

      return
    end
    
    
    def init_socket
      logger.info( "init_socket")
      
      if ! @enable 
        return
      end

      begin
        @socket = File.new( @file)
      rescue Exception => ex
        logger.warn( "init_socket::ex: #{ex}")
      end
        
      
      return
    end
    
    
    def read_message
      logger.info( "read_message")

      if @first_read
        sleep @offset
        @first_read = false
      end

      sleep @delay

      @counter += 1

      if (@counter % 1000) == 0
        logger.error( "counter: #{@counter}")
      end

      data = read_data

      message = parse_message( data)

      logger.debug( "read_message: message: #{message} #{message.nil?}")

      return message
    end

    
    def read_data
      logger.info( "read_data")

      line = nil

      begin
        line = @socket.gets

        # if nil, an exception is raised (and reinit is started)
        line.chomp! if line
        logger.debug( "readData::line: -#{line}-")
        
      rescue Exception => ex
        logger.warn( "read_data::ex: " + ex)
      end until line !~ /^#/
        
      return line
    end
    

    def parse_message( data)
      logger.info( "parse_message( data)")
      
      logger.debug( "data: #{data}")

      if data.nil?
        return nil
      end
      
      # todo more robust message parsing 

      message = nil
      headerIndex = data.index( ':')
      
      if !headerIndex.nil?
        
        message = APRSMessage.new
        # header
        header = data[0...headerIndex]
        
        sourceIndex = header.index( '>')
        
        if sourceIndex != nil 
          message.source = APRSCall.parse( header[0...sourceIndex]).to_s
        end

        # path
        message.path = Array.new
        path = header[sourceIndex+1...header.length].split( ',')
        message.destination = APRSCall.parse( path[0]).to_s

        path.delete_at( 0)
        
        path.each{ |entry|
          call = APRSCall.parse( entry)
          message.path << call.to_s if call
        }

        # body
        body = data[headerIndex+1...data.length]
        message.payload = body
      end

      return message
    end

    
    def write_message( message) 
      logger.info( "write_message( message)")
      
      return
    end

  end

end
